from util import *
import os
from pyinotify import WatchManager, Notifier, ThreadedNotifier, EventsCodes, ProcessEvent, ALL_EVENTS
import time
import base64

FLAGS = ALL_EVENTS

class FileEventProcessor(ProcessEvent):
    def __init__(self, proxy):
        self.proxy = proxy

    def process_IN_ACCESS(self, event):
        f = os.path.join(event.path, event.name)
        #print "Access %s" % (f)
        time.sleep(0.1)
        f2 = os.path.dirname(f)
        lastLog = getLastWatch(f2)
        #print lastLog

    def process_IN_ATTRIB(self, event):
        f = os.path.join(event.path, event.name)
        print "Attrib %s" % (f)
        time.sleep(0.1)
        f2 = os.path.dirname(f)
        lastLog = getLastWatch(f2)
        #print lastLog
        self.proxy.handleModify(f, lastLog)

    def process_IN_CLOSE(self, event):
        f = os.path.join(event.path, event.name)
        #print "Close %s" % (f)
        time.sleep(0.1)
        f2 = os.path.dirname(f)
        lastLog = getLastWatch(f2)
        #print lastLog
        #self.proxy.handleModify(f, lastLog)

    def process_IN_CREATE(self, event):
        f = os.path.join(event.path, event.name)
        print "Create %s" % (f)
        time.sleep(0.1)
        f2 = os.path.dirname(f)
        lastLog = getLastWatch(f2)
        #print lastLog

    def process_IN_DELETE(self, event):
        f = os.path.join(event.path, event.name)
        print "Delete %s" % (f)
        time.sleep(0.1)
        f2 = os.path.dirname(f)
        lastLog = getLastWatch(f2)
        #print lastLog
        self.proxy.handleModify(f, lastLog)

    def process_IN_DELETE_SELF(self, event):
        self.process_IN_DELETE(event)

    def process_IN_MODIFY(self, event):
        f = os.path.join(event.path, event.name)
        print "Modify %s" % (f)
        time.sleep(0.1)
        f2 = os.path.dirname(f)
        lastLog = getLastWatch(f2)
        #print lastLog
        self.proxy.handleModify(f, lastLog)

    def process_IN_MOVE_SELF(self, event):
        self.process_IN_DELETE(event)

    def process_IN_MOVED_FROM(self, event):
        self.process_IN_DELETE(event)

    def process_IN_MOVED_TO(self, event):
        self.process_IN_CREATE(event)

    def process_IN_OPEN(self, event):
        f = os.path.join(event.path, event.name)
        #print "Open %s" % (f)
        time.sleep(0.1)
        f2 = os.path.dirname(f)
        lastLog = getLastWatch(f2)
        #print lastLog

    def process_default(self, event):
        f = os.path.join(event.path, event.name)
        #print "Default %s" % (f)
        time.sleep(0.1)
        f2 = os.path.dirname(f)
        lastLog = getLastWatch(f2)
        #print lastLog

class FileProxy:
    def __init__(self, config):
        self.running = True
        self.crashes = 0
        files = config['FILES']
        self.ppid = config['PID']
        self.files = []
        self.backups = []
        self.watched = []
        wm = WatchManager()
        self.notifyProcessor = FileEventProcessor(self)
        self.notifier = Notifier(wm, self.notifyProcessor)
        for f in files:
            f1 = os.path.realpath(f)
            if f[-1] != '/':
                f2 = os.path.dirname(f1)
            else:
                f2 = f1
            f2 = f2
            wm.add_watch(f2, FLAGS, rec=True)
            watch(f2)
            self.watched.append(f2)
            self.backup(f1)
            self.files.append(f1)

    def backup(self, f):
        print "BACKUP %s" % f
        code = base64.b64encode(f, "_-")
        if not code in self.backups:
            self.backups.append(code)
        subprocess.call(["zip", "-r", "%s.zip" % (code), f])
        pass

    def restore(self, f):
        print "RESTORE %s" % f
        code = base64.b64encode(f, "_-")
        subprocess.call(["unzip", "-o", "%s.zip" % (code), "-d", "/"])
        pass

    def handleModify(self, f, log):
        #print log
        if len(log) < 1:
            return
        if f in self.files or os.path.dirname(f) in self.files:
            rf = f
            if not f in self.files:
                rf = os.path.dirname(f)
            if ("ppid=%s" % (self.ppid)) in log[-1]:
                self.backup(rf)
            else:
                self.restore(rf)

    def spArgs(self):
        return {}

    def run(self):
        while self.running:
            self.notifier.process_events()
            if self.notifier.check_events():
                self.notifier.read_events()

    def reportCrash(self):
        self.crashes += 1

    def clean(self):
        self.notifier.stop()
        for f in self.watched:
            unwatch(f)
        for f in self.backups:
            os.remove(f + ".zip")
