import socket
import sys
import subprocess
import select 
import errno
import time
import logging
import logging.config
import re

logging.basicConfig(level=logging.DEBUG, format='%(asctime)s: %(message)s')

class StdFilter:
    logger = logging.getLogger('stdLogger')
    logger.info('setting up filter!')

    def __init__(self, pin, cin):
        self.pin = pin
        self.cin = cin
        self.pin_type = type(pin)
        self.cin_type = type(cin)
        self.banned = {}
        self.max_len = (None, None)
        self.inp = ''

    def on_recv(self):
        inp = ''
        #logging.info('Reading from ' + str(self.pin))
        if self.pin_type == file:
            inp = []
            try:
                line = self.pin.readline()
                stripped = line.strip()
                inp.append(stripped)
                inp = '\n'.join(inp)
            except:
                pass
        #logging.info('Received: ' + inp)
        self.inp = inp
        return inp
        
    def send(self, inp):
        if inp != '':
            data = self.filter(inp)
            if data == None:
                return
            if self.cin_type == file:
                self.cin.write(data + '\n')
            else:
                self.cin.sendall(data + '\n')

    def filter(self, inp):
        # filter/sanitize inp
        #logging.info('sanitizing input: ' + inp)
        if inp in self.banned and self.banned[inp] > 1:
            logging.info(inp + ' is a banned input.  Not sending.')
            return None
        if self.max_len != (None, None) and len(inp) > self.max_len[0]:
            logging.info(inp + ' is too long based on previous inputs.  Not sending.')
            return None
        return inp
    
    def forward(self):
        while True:
            data = self.on_recv();
            if not data:
                return
            self.send(data)

    def ban(self):
        if self.inp != '':
            self.banned[self.inp] = self.banned.get(self.inp, 0) + 1
            logging.info("Banned list updated: " + str(self.banned.items()))
 
    def max_allowed(self):
        inp_len = len(self.inp)
        if inp_len > 256:
            if self.max_len == (None, None):
                self.max_len = (inp_len, 1)
            else:
                self.max_len = (((1./(self.max_len[1]+1))*inp_len + (1.-(1./(self.max_len[1]+1)))*self.max_len[0]), (self.max_len[1]+1))
        logging.info("Setting max input length to " + str(self.max_len[0]))

class StdProxy:
    logger = logging.getLogger('stdLogger')

    def __init__(self, config):
        self.running = True
        self.p, self.c = socket.socketpair(socket.AF_UNIX, socket.SOCK_DGRAM)
        self.crashes = 0
        self.stdin = sys.stdin
        self.stdin_filter = StdFilter(self.stdin, self.p)


    def spArgs(self):
        return {'stdin': self.c}
#        return {'stdin': subprocess.PIPE}

    def run(self):
        # Create proxy on sockets
        while self.running:
            self.stdin_filter.forward()
            time.sleep(5)

    def reportCrash(self):
        self.crashes += 1
        self.stdin_filter.ban()
        self.p, self.c = socket.socketpair(socket.AF_UNIX, socket.SOCK_DGRAM)
        self.stdin_filter.cin = self.p
        self.stdin_filter.max_allowed()

    def clean(self):
        # Destroy socksts
        self.p.close()
        self.c.close()


# For testing
#if __name__=="__main__":
#    logging.info('Test logging')
#    proxy = StdProxy({})
#    spArgs = proxy.spArgs()
#    subprocess.Popen(['python', './other_apps/pipe2.py'], **spArgs)
#    proxy.run()
